@@ -2,7 +2,7 @@
 
 /**
   * SquirrelMail Spam Buttons Plugin
-  * Copyright (c) 2005-2008 Paul Lesniewski <paul@squirrelmail.org>,
+  * Copyright (c) 2005-2009 Paul Lesniewski <paul@squirrelmail.org>,
   * Licensed under the GNU GPL. For full terms see the file COPYING.
   *
   * @package plugins
@@ -95,12 +95,25 @@ function sb_button_action_do($args)
 
 
    $passed_ent_id = 0;
-   if (!sqGetGlobalVar('location', $location, SQ_FORM)) $location = php_self();
-   sqGetGlobalVar('passed_id',       $passed_id,      SQ_FORM);
    sqGetGlobalVar('passed_ent_id',   $passed_ent_id,  SQ_FORM);
-   sqGetGlobalVar('msg',             $msg,            SQ_FORM);
    sqGetGlobalVar('REQUEST_METHOD',  $method,         SQ_SERVER);
 
+   if (sqGetGlobalVar('location', $location, SQ_FORM))
+      $location = htmlspecialchars($location);
+   else
+      $location = php_self();
+
+   if (sqGetGlobalVar('passed_id', $passed_id, SQ_FORM))
+      // fix for Dovecot UIDs can be bigger than normal integers
+      $passed_id = (preg_match('/^[0-9]+$/', $passed_id) ? $passed_id : '0');
+
+   if (sqGetGlobalVar('msg', $msg, SQ_FORM))
+      // fix for Dovecot UIDs can be bigger than normal integers
+      if (is_array($msg)) foreach ($msg as $i => $messageID)
+         $msg[$i] = (preg_match('/^[0-9]+$/', $messageID) ? $messageID : '0');
+      else
+         $msg = (preg_match('/^[0-9]+$/', $msg) ? $msg : '0');
+
 
    // determine if the report was done from the message view screen
    //
@@ -240,7 +253,10 @@ function sb_button_action_do($args)
       //
       if ($result)
       {
-         if (count($msg) < 2 && !empty($custom_button_success_singular))
+         if (!empty($custom_button_success_singular)
+          && !empty($custom_button_success_plural))
+            $note = ngettext($custom_button_success_singular, $custom_button_success_plural, count($msg));
+         else if (count($msg) < 2 && !empty($custom_button_success_singular))
             $note = _($custom_button_success_singular);
          else if (count($msg) > 1 && !empty($custom_button_success_plural))
             $note = _($custom_button_success_plural);
@@ -1131,10 +1147,19 @@ function sb_button_action_do($args)
                    . (!empty($account) ? "&account=$account" : '')
                    . (!empty($startMessage) ? "&startMessage=$startMessage" : '');
          if (check_sm_version(1, 5, 2)) $key = FALSE;
+//LEFT OFF HERE -- debugging UW 
+//sm_print_r('$imapConnection contains:', $imapConnection);
+//sqimap_logout($imapConnection);
          if (!is_resource($imapConnection))
             $imapConnection = sqimap_login($username, $key, $imapServerAddress, $imapPort, 0);
+//echo '$mbx_response now contains:<br />';
+//sm_print_r($mbx_response);
          if (empty($mbx_response))
+//{
             $mbx_response = sqimap_mailbox_select($imapConnection, $mailbox);
+//echo 'And now $mbx_response contains:<br />';
+//sm_print_r($mbx_response);
+//}
 
          if (check_sm_version(1, 5, 2))
          {
@@ -1144,10 +1169,14 @@ function sb_button_action_do($args)
          else
          {
             sqimap_msgs_list_delete($imapConnection, $mailbox, $msg);
+//echo "Finished deleting; now expunge if necessary...<br />";
             if ($auto_expunge) 
             {
+//echo "Expunging...<br />";
                $cnt = sqimap_mailbox_expunge($imapConnection, $mailbox, true);
             }
+//echo "Finished<br />";
+//exit;
          }
 
          
@@ -1414,7 +1443,7 @@ function report_by_email($destination, $msg, $passed_ent_id,
           $spam_report_smtpPort, $spam_report_useSendmail, 
           $spam_report_smtp_auth_mech, $spam_report_use_smtp_tls,
           $smtpServerAddress, $smtpPort, $useSendmail, $smtp_auth_mech, 
-          $use_smtp_tls, $sb_debug, $username, $domain;
+          $use_smtp_tls, $sb_debug, $data_dir, $username, $domain;
    $at_sign = '@';
 
    spam_buttons_init();
@@ -1429,8 +1458,9 @@ function report_by_email($destination, $msg, $passed_ent_id,
       $user = $username;
       $dom = $domain;
    }
-   $destination = str_replace(array('###EMAIL_ADDRESS###', '###USERNAME###', '###DOMAIN###'), 
-                              array($user . $at_sign . $dom, $user, $dom), 
+   $email_address = getPref($data_dir, $username, 'email_address');
+   $destination = str_replace(array('###EMAIL_PREF###', '###EMAIL_ADDRESS###', '###USERNAME###', '###DOMAIN###'), 
+                              array($email_address, $user . $at_sign . $dom, $user, $dom), 
                               $destination);
 
 
@@ -1572,7 +1602,7 @@ function report_by_email($destination, $msg, $passed_ent_id,
 function report_by_shell_command($command, $msg, $passed_ent_id)
 {
 
-   global $attachment_dir, $username, $domain, $sb_debug;
+   global $attachment_dir, $data_dir, $username, $domain, $sb_debug;
    $at_sign = '@';
 
    spam_buttons_init();
@@ -1592,8 +1622,9 @@ function report_by_shell_command($command, $msg, $passed_ent_id)
       $user = $username;
       $dom = $domain;
    }
-   $command = str_replace(array('###EMAIL_ADDRESS###', '###USERNAME###', '###DOMAIN###'), 
-                          array($user . $at_sign . $dom, $user, $dom), 
+   $email_address = getPref($data_dir, $username, 'email_address');
+   $command = str_replace(array('###EMAIL_PREF###', '###EMAIL_ADDRESS###', '###USERNAME###', '###DOMAIN###'), 
+                          array($email_address, $user . $at_sign . $dom, $user, $dom), 
                           $command);
 
 
@@ -1828,12 +1859,12 @@ function spam_buttons_findNextMessage($passed_id)
 
 
 
-// The move and copy function names change as of 1.5.0 to be what 
+// The move and copy function names change as of 1.4.18 to be what 
 // they say they are, so use those if we are at the right version.
 //
-// For 1.4.x, there is no copy function, so we use a modified
-// version of the move function from 1.4.11 (which, ironically
-// is mis-labelled "copy")
+// For versions less than 1.4.18, there is no copy function, so we
+// use a modified version of the move function from 1.4.11 (which,
+// ironically is mis-labelled "copy")
 //
 // Also, if configured to do so, the target folder will be created
 // first if it does not exist.
@@ -1843,7 +1874,7 @@ function spam_buttons_sqimap_msgs_list_copy($imap_stream, $id, $mailbox) {
    spam_buttons_auto_create_folder($imap_stream, $mailbox);
 
 
-   if (check_sm_version(1, 5, 0))
+   if (check_sm_version(1, 4, 18))
       return sqimap_msgs_list_copy($imap_stream, $id, $mailbox);
 
 
@@ -1852,12 +1883,16 @@ function spam_buttons_sqimap_msgs_list_copy($imap_stream, $id, $mailbox) {
    global $uid_support;
    $msgs_id = sqimap_message_list_squisher($id);
    $read = sqimap_run_command ($imap_stream, "COPY $msgs_id \"$mailbox\"", true, $response, $message, $uid_support);
+   if ($response == 'OK')
+      return true;
+   else
+      return false;
 
 }
 
 
 
-// the move and copy function names change as of 1.5.0 to be what they say they are
+// the move and copy function names change as of 1.4.18 to be what they say they are
 //
 // Also, if configured to do so, the target folder will be created
 // first if it does not exist.
@@ -1867,15 +1902,13 @@ function spam_buttons_sqimap_msgs_list_move($imap_stream, $id, $mailbox) {
    spam_buttons_auto_create_folder($imap_stream, $mailbox);
 
 
-   if (check_sm_version(1, 5, 0))
+   if (check_sm_version(1, 4, 18))
       return sqimap_msgs_list_move($imap_stream, $id, $mailbox);
 
 
-   else
-
-      // mis-labelled function name in 1.4.x
-      //
-      return sqimap_msgs_list_copy($imap_stream, $id, $mailbox);
+   // mis-labelled function name in versions less than 1.4.18
+   //
+   return sqimap_msgs_list_copy($imap_stream, $id, $mailbox);
 
 }
 
